<?php

namespace Drupal\estimated_read_time\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'estimated_read_time' field type.
 *
 * @FieldType(
 *   id = "estimated_read_time",
 *   label = @Translation("Read Time"),
 *   module = "estimated_read_time",
 *   category = "Read Time",
 *   description = @Translation("Estimates the read time of an entity."),
 *   default_widget = "estimated_read_time_default",
 *   default_formatter = "estimated_read_time_text"
 * )
 */
class EstimatedReadTimeItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'minutes' => [
          'type' => 'int',
          'unsigned' => TRUE,
          'size' => 'tiny',
          'not null' => FALSE,
        ],
        'seconds' => [
          'type' => 'int',
          'unsigned' => TRUE,
          'size' => 'tiny',
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'view_mode' => [],
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityBase $entity */
    $entity = $form['#entity'];

    /** @var \Drupal\Core\Entity\EntityDisplayRepository $entityDisplayRepository */
    $entityDisplayRepository = \Drupal::service('entity_display.repository');
    $viewModes = $entityDisplayRepository->getViewModeOptionsByBundle($entity->getEntityTypeId(), $entity->bundle());

    $element['view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('View Mode'),
      '#description' => $this->t('The view mode to use to calculate the read time.'),
      '#options' => $viewModes,
      '#default_value' => $this->getSetting('view_mode'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $minutes = random_int(1, 30);
    $seconds = random_int(1, 59);

    $values['minutes'] = $minutes;
    $values['seconds'] = $seconds;

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $minutes = $this->get('minutes')->getValue();
    $seconds = $this->get('seconds')->getValue();
    return ($minutes === NULL || $minutes === '') && ($seconds === NULL || $seconds === '');
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['minutes'] = DataDefinition::create('integer')
      ->setLabel(t('Minutes'));
    $properties['seconds'] = DataDefinition::create('integer')
      ->setLabel(t('Seconds'));

    return $properties;
  }

}
